use strict;
use warnings;

use Test::More tests => 6;
use_ok('Tickit::Widget::Table');
use_ok('Tickit::Widget::Table::Column');
use_ok('Tickit::Widget::Table::Row');
use_ok('Tickit::Widget::Table::Cell');
use_ok('Tickit::Widget::Table::HeaderRow');
use_ok('Tickit::Widget::Table::HeaderCell');

